
/*
     File:       DICOM_Toolkit_Types.h

     Contains:   Escape DICOM Toolkit Type Definitions.

     Copyright:  (c)2002-2004 by Escape Information Services, all rights reserved.

     Bugs?       For support or bug reports please send email to support@escape.gr
 
*/
#ifndef __DICOM_TOOLKIT_TYPES__
#define __DICOM_TOOLKIT_TYPES__

#if defined(_WIN32) && (_WIN32 == 1) && !defined(__MACTYPES__)
	#include <WinDef.h>
	#define noErr kDTkNoErr
	typedef unsigned char       Boolean;
	typedef unsigned char       UInt8;
	typedef signed char         SInt8;
	typedef unsigned short      UInt16;
	typedef signed short        SInt16;
	typedef unsigned long       UInt32;
	typedef signed long         SInt32;
	typedef float               Float32;
    typedef double		        Float64;
	typedef char *              Ptr;
	typedef Ptr *               Handle;
    typedef long				Size;
    typedef SInt16              OSErr;
	typedef unsigned long       FourCharCode;	
	typedef FourCharCode        OSType;
 	#define FOUR_CHAR_CODE(x) (x)
	#include <stdint.h> /* intptr_t */
	#ifndef __COMPONENTS__
		typedef intptr_t ComponentInstance;
	#endif
    #if defined(_MSC_VER) && !defined(__MWERKS__) && defined(_M_IX86)
      typedef   signed __int64                SInt64;
        typedef unsigned __int64                UInt64;
    #else
      typedef   signed long long              SInt64;
        typedef unsigned long long              UInt64;
    #endif
	#define HiWord(x) ((short)((long)(x) >> 16))
	#define LoWord(x) ((short)(x))
#else
	#ifndef __MACH__
		#include <MacTypes.h>
		#include <Components.h>
	#endif
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_IMPORT
#pragma import on
#endif

#if PRAGMA_STRUCT_ALIGN
    #pragma options align=mac68k
#elif PRAGMA_STRUCT_PACKPUSH
    #pragma pack(push, 2)
#elif PRAGMA_STRUCT_PACK
    #pragma pack(2)
#endif

#if PRAGMA_ENUM_ALWAYSINT
    #if defined(__fourbyteints__) && !__fourbyteints__ 
        #define __DICOM_TOOLKIT_TYPES__RESTORE_TWOBYTEINTS
        #pragma fourbyteints on
    #endif
    #pragma enumsalwaysint on
#elif PRAGMA_ENUM_OPTIONS
    #pragma option enum=int
#elif PRAGMA_ENUM_PACK
    #if __option(pack_enums)
        #define __DICOM_TOOLKIT_TYPES__RESTORE_PACKED_ENUMS
        #pragma options(!pack_enums)
    #endif
#endif

/*!
    @header DICOM_Toolkit_Types.h
    @abstract This header includes the basic type definitions used by the rest of the DICOM Toolkit API
*/


#pragma mark enum DTkErrorCode

/*!
    @enum DTkErrorCode
    @abstract Miscellaneous error codes returned by the DICOM Toolkit's functions.
    @constant kDTkNoErr                         Signifies that the function returned successfully
    @constant kDTkBadElementRefErr              An invalid element reference number was entered
    @constant kDTkIndexOutOfRangeErr            The index entered is out of range
    @constant kDTkNullDataPtr		            The pointer passed to receive the data is null
    @constant kDTkBadMultiplicityErr            Invalid item multiplicity
    @constant kDTkBadElemTypeErr                Invalid element type
    @constant kDTkInvalidParentTypeErr          Invalid type of parent element
    @constant kDTkBadValueRecErr                Invalid value record
    @constant kDTkElementNotFoundErr            The element was not found
    @constant kDTkElementValueNotFoundErr       The data element exists but it has no value
	@constant kDTkInvalidVRErr                  An invalid value representation was encountered.
	@constant kDTkExpectedNullOutputHandleErr   This error is returned by functions that return a data in a handle to which a pointer is passed. 
                                                The value of the handle should be NULL so that it is not inadvertently clobbered. 
    @constant kDTkInvalidVRTypeErr              An invalid value representation type was encountered.
    @constant kDTkInvalidEndianErr              An invalid (or no) endian was specified.
    @constant kDTkInvalidTransferSyntaxErr		An invalid transfer syntax was encountered
	
    @constant kDTkBadBlitterStateErr	        Invalid VOI blitter state. This error is emitted by DTkCodecVOIBlitterInit
                                                when it is called before the image data have been decompressed.
	@constant kDTkInvalidPixelValueBoundsErr    The decompressor attempted to use invalid min/max pixel values.
	@constant kDTkDisplayPixelBoundsNotFoundErr The decompressor attempted to retrieve user-defined pixel bounds which 
                                                have not been previously set by DTkCodecSetDisplayPixelBounds.
	@constant kDTkNoMaskPixelMapWasSetErr       A masking operation was attempted but bo mask pixel data were found.
	@constant kDTkMaskPixelMapNotCompatibleErr  A masking operation was attempted with mask pixel data incompatible to image pixel data

	@constant kDTkNoImageDataFoundErr			The DICOM graphics importer or movie importer did not find image 
	                                            data in the input data. The data may otherwise be valid DICOM data.
	@constant kDTkNoOverlayDataFoundErr			The DICOM graphics importer or movie importer did not find overlay data 
	                                            required for the operation.

*/
typedef enum
{
    kDTkNoErr                         =  0,

    kDTkBadElementRefErr              = -40001,
    kDTkIndexOutOfRangeErr            = -40002,
    kDTkNullDataPtr		              = -40003,
    kDTkBadMultiplicityErr            = -40004,
    kDTkBadElemTypeErr                = -40005,
    kDTkInvalidParentTypeErr          = -40006,
    kDTkBadValueRecErr                = -40007,
    kDTkElementNotFoundErr            = -40008,
    kDTkElementValueNotFoundErr       = -40009,
    kDTkInvalidVRErr                  = -40010,
    kDTkExpectedNullOutputHandleErr   = -40011,
    kDTkInvalidVRTypeErr              = -40012,
    kDTkInvalidEndianErr              = -40013,
    kDTkInvalidTransferSyntaxErr      = -40014,

    kDTkBadBlitterStateErr            = -41001,
    kDTkInvalidPixelValueBoundsErr    = -41002,
    kDTkDisplayPixelBoundsNotFoundErr = -41003,
    kDTkNoMaskPixelMapWasSetErr		  = -41004,
    kDTkMaskPixelMapNotCompatibleErr  = -41005,

    kDTkNoImageDataFoundErr			  = -42001,
    kDTkNoOverlayDataFoundErr		  = -42002
} DTkErrorCode;


/*!
    @const kDTkUnknown
    @abstract Four character code kDTkUnknown
    @discussion Four character code used in various enumerations throughout the toolkit to signify an unknown value
    @constant kDTkUnknown Identifies an unknown value
*/
enum
{
    kDTkUnknown = FOUR_CHAR_CODE('UNKN')
};

/*!
    @typedef DTkElementRef
    @abstract References a DICOM data element
*/
typedef long DTkElementRef;
typedef DTkElementRef  *DTkElementRefPtr;

#pragma mark -
#pragma mark enum DTkNumericValueType

/*!
    @enum DTkNumericValueType
    @abstract Type definition of enumeration of allowed values of DICOM numeric value types.
    @constant kDTkNumericValueTypeSInt8            The value is of type SInt8 (signed 8 bit integer)
    @constant kDTkNumericValueTypeUInt8            The value is of type UInt8 (unsigned 8 bit integer)
    @constant kDTkNumericValueTypeSInt16           The value is of type SInt16 (signed 16 bit integer, corresponds to the Signed Short [SS] DICOM VR)
    @constant kDTkNumericValueTypeUInt16           The value is of type UInt16 (unsigned 16 bit integer, corresponds to the Unsigned Short [US] DICOM VR)
    @constant kDTkNumericValueTypeSInt32           The value is of type SInt32 (signed 32 bit integer, corresponds to the Signed Long [SL] DICOM VR)
    @constant kDTkNumericValueTypeUInt32           The value is of type UInt32 (unsigned 32 bit integer, corresponds to the Unsigned Long [UL] DICOM VR)
    @constant kDTkNumericValueTypeFloat32          The value is of type Float32 (32 bit floating point number, corresponds to the Floating Point Single [FL] DICOM VR)
    @constant kDTkNumericValueTypeFloat64          The value is of type Float64 (64 bit floating point number, corresponds to the Floating Point Double [FD] DICOM VR)
    @constant kDTkNumericValueType1ByteItemVectPtr The value is a pointer to vector of single byte values that can be interpreted as signed or unsigned quantities 
                                                   (corresponds to the Other Byte String [OB] DICOM VR)
    @constant kDTkNumericValueType2ByteItemVectPtr The value is a pointer to vector of two byte values that can be interpreted as signed or unsigned quantities
                                                   (corresponds to the Other Word String [OW] DICOM VR)
    @constant kDTkNumericValueType4ByteItemVectPtr The value is a pointer to vector of four byte values that can be interpreted as signed or unsigned quantities
                                                   (corresponds to the Other Float String [OF] DICOM VR)
	@constant kDTkNumericValueType2UInt16		   The value is a vector of two integers of type UInt16 (corresponds to the Attribute Tag [AT] DICOM VR)
    @constant kDTkNumericValueTypeNoValue          No value exists
    @constant kDTkNumericValueTypeNonNumeric       The value is of a non numeric type
    @constant kDTkNumericValueTypeUnknown          Identifies an unknown value type
*/
typedef enum 
{
    kDTkNumericValueTypeSInt8,
    kDTkNumericValueTypeUInt8,
    kDTkNumericValueTypeSInt16,
    kDTkNumericValueTypeUInt16,
    kDTkNumericValueTypeSInt32,
    kDTkNumericValueTypeUInt32,
    kDTkNumericValueTypeFloat32,
    kDTkNumericValueTypeFloat64,
    kDTkNumericValueTypeNoValue,
    kDTkNumericValueTypeNonNumeric,
    kDTkNumericValueType1ByteItemVectPtr,
    kDTkNumericValueType2ByteItemVectPtr,
    kDTkNumericValueType4ByteItemVectPtr,
    kDTkNumericValueType2UInt16,
    kDTkNumericValueTypeUnknown    = kDTkUnknown
} DTkNumericValueType;


#pragma mark union DTkNumericValue

/*!
    @typedef DTkNumericValue
    @abstract This is a type definition of a union of the fields in the various possible types a numeric DICOM element take.
    @field value_SC    A value of type SInt8 (signed 8 bit integer)
    @field value_UC A value of type UInt8 (unsigned 8 bit integer)
    @field value_SS A value of type SInt16 (signed 16 bit integer)
    @field value_US A value of type UInt16 (unsigned 16 bit integer)
    @field value_SL A value of type SInt32 (signed 32 bit integer)
    @field value_UL A value of type UInt32 (unsigned 32 bit integer)
    @field value_FL A value of type Float32 (32 bit floating point number)
    @field value_FD A value of type Float64 (64 bit floating point number)
    @field value_OB A handle to a string (vector) of single byte values cooresponding to the OB DICOM value representation. The actual values may be interpreted as signed or unsigned.
    @field value_OW A handle to a string (vector) of two byte values cooresponding to the OW DICOM value representation. The actual values may be interpreted as signed or unsigned.
    @field value_OF A handle to a string (vector) of four byte values cooresponding to the OF DICOM value representation.
    @field value_AT A handle to a vector of two UInt16 (unsigned 16 bit integer) cooresponding to the AT DICOM value representation.
*/
typedef union 
{
    SInt8     value_SC;
    UInt8     value_UC;
    SInt16    value_SS;
    UInt16    value_US;
    SInt32    value_SL;
    UInt32    value_UL;
    Float32   value_FL;
    Float64   value_FD;
    UInt8   **value_OB;
    UInt16  **value_OW;
    Float32 **value_OF;
    UInt16    value_AT[2];
} DTkNumericValue, *DTkNumericValuePtr;


#pragma mark struct DTkNumericValueRecord

/*!
    @typedef DTkNumericValueRecord
    @abstract Type definition of a structure consisting of a value field and an associated type specifier field
    @field valueType    The type of the field 'value'
    @field value        The value
*/
typedef struct
{
    DTkNumericValueType  valueType;
    DTkNumericValue value;
} DTkNumericValueRecord, *DTkNumericValueRecordPtr;

#pragma mark -
#pragma mark struct DTkElemRefArray

/*!
    @typedef DTkElemRefArray
    @abstract Structure of array of DTkElementRef
    @field numElements Number of elements in the array
    @field elemRef The array of items of type  DTkElementRef
*/

typedef struct
{
    UInt32 numElements;
    DTkElementRef elemRef[1];
} DTkElemRefArray, *DTkElemRefArrayPtr, **DTkElemRefArrayHandle, ***DTkElemRefArrayHandlePtr;

#pragma mark enum DTkElementStatus

/*!
    @enum DTkElementStatus
    @abstract Enumeration of allowed values of the status of a DICOM tag
    @constant kDTkElementStatusRetired Identifies a retired DICOM tag
    @constant kDTkElementStatusUnknown Identifies an active DICOM tag
    @constant kDTkElementStatusUnknown Identifies a DICOM tag with an unknown status
*/
typedef enum
{
    kDTkElementStatusRetired,
    kDTkElementStatusActive,
    kDTkElementStatusUnknown = kDTkUnknown
} DTkElementStatus;


#pragma mark struct DTkElementTag
/*!
    @typedef DTkElementTag
    @abstract Struct identifying a DICOM tag
    @field group The DICOM tag's group number
    @field element The DICOM tag's element number
*/
typedef struct
{
    UInt16 group;
    UInt16 element;
} DTkElementTag, *DTkElementTagPtr;


#pragma mark struct DTkElemTagArray
/*!
    @typedef DTkElemTagArray
    @abstract Struct identifying a DICOM tag array
    @field numElements Number of elements in the array
    @field elemTag An array of pointers to DTkElementTag structs
*/
typedef struct
{
    UInt32 numElements;
    DTkElementTagPtr elemTag;
} DTkElemTagArray, *DTkElemTagArrayPtr;


#pragma mark enum DTkVR

/*!
    @enum DTkVR
    @abstract Extended enumeration of DICOM Value Representations (DICOM standard, part 5, section 6.2).
    @discussion This enumeration is used in conjunction with the DICOM Dictionary 
                and includes all the Value Representations (VRs) defined in the DICOM standard 
                along with some that deal with the unavailability of VRs for certain tags in the dictionary
                (e.g. for retired items), VR ambiguities for others and toolkit defined VRs used with certain 
                data elements which are used as delimiters with sequences.
    @constant kDTkVR_Empty This value is used to initialize variables (defined by this toolkit)
    @constant kDTkVR_ApplicationEntity 'Application Entity', 16 bytes max
    @constant kDTkVR_AgeString 'Age String', 4 bytes fixed 
    @constant kDTkVR_AttributeTag  Attribute Tag 'AT', 4 bytes fixed
    @constant kDTkVR_CodeString  Code String 'CS', 16 bytes max
    @constant kDTkVR_Date  Date 'DA', 8 or 10 bytes fixed
    @constant kDTkVR_DecimalString  Decimal String 'DS', 16 bytes max
    @constant kDTkVR_DateTime  Date Time 'DT', 26 bytes max
    @constant kDTkVR_FloatingPointSingle  Floating Point Single 'FS', 4 bytes fixed
    @constant kDTkVR_FloatingPointDouble  Floating Point Double 'FD', 8 bytes fixed
    @constant kDTkVR_IntegerString  Integer String 'IS', 12 bytes max
    @constant kDTkVR_IntegerString  Long String 'LO', 64 chars max
    @constant kDTkVR_LongText  Long Text 'LT', 10240 chars max
    @constant kDTkVR_OtherByteString  Other Byte String 'OB', unlimited even byte length, endian insensitive
    @constant kDTkVR_OtherFloatString  Other Float String 'OF', string of 32 bit words, endian sensitive, 2^32 - 4 maximum length
    @constant kDTkVR_OtherWordString  Other Word String 'OW', unlimited length of 16 bit words, endian sensitive
    @constant kDTkVR_PersonName  Person Name 'PN', 64 chars max
    @constant kDTkVR_ShortString  Short String 'SH', 16 chars max
    @constant kDTkVR_SignedLong  Signed Long 'SL', 4 bytes fixed
    @constant kDTkVR_SequenceOfItems  Sequence Of Items 'SQ', Special VR used for nested data
    @constant kDTkVR_SignedShort  Signed Short 'SS', 2 bytes fixed
    @constant kDTkVR_ShortText  Short Text 'ST', 1024 chars max
    @constant kDTkVR_Time  Time 'TI', 16 bytes max
    @constant kDTkVR_UniqueIdentifier  Unique Identifier 'UI', 64 bytes max
    @constant kDTkVR_UnsignedLong  Unsigned Long 'UL', 4 bytes fixed
    @constant kDTkVR_Unknown  Unknown 'UN', any valid length of other VR
    @constant kDTkVR_UnsignedShort  Unsigned Short 'US', 2 bytes fixed
    @constant kDTkVR_UnlimitedText  Unlimited Text 'UL', up to 2^32 - 2 chars
    @constant kDTkVR_USorSS  'US' or 'SS' (defined by this toolkit)
    @constant kDTkVR_USorSSorOW  'US' or 'SS' or 'OW' (defined by this toolkit)
    @constant kDTkVR_OWorOB  'OW' or 'OB'  (defined by this toolkit)
    @constant kDTkVR_SequenceItem This toolkit-defined VR is reserved for use with the "Item Data Element" with tag (0xFFFE,0xE000). 
        Each Item of a Data Element of Value Representation SQ is encoded as an "Item Data Element". See DICOM Standard, part 5, section 7.5.1.
    @constant kDTkVR_SeqItemDelimDataElement This toolkit-defined VR is reserved for use with the "Item Delimitation Item Data Element" with tag (0xFFFE,0xE00D).
        Such data elements are used to delimit sequence items of undefined length. See DICOM Standard, part 5, section 7.5.1.
    @constant kDTkVR_SeqDataElementEnd This toolkit-defined VR is reserved for use with the "Sequence Delimitation Item" with tag (0xFFFE,0xE0DD).
        This item is used to delimit a "Sequence of Items" encapsulated in a Data Element of Value Representation SQ and of undefined length. 
        The Sequence Delimitation Item is included after the last Item in the sequence. See DICOM Standard, part 5, section 7.5.2.
    @constant kDTkVR_NODICTENTRY  Is used for DICOM tags for which the dictionary does not specify a VR (defined by this toolkit).

*/
typedef enum
{
    kDTkVR_Empty                   = FOUR_CHAR_CODE('VR  '),
    kDTkVR_ApplicationEntity       = FOUR_CHAR_CODE('VRAE'),
    kDTkVR_AgeString               = FOUR_CHAR_CODE('VRAS'),
    kDTkVR_AttributeTag            = FOUR_CHAR_CODE('VRAT'),
    kDTkVR_CodeString              = FOUR_CHAR_CODE('VRCS'),
    kDTkVR_Date                    = FOUR_CHAR_CODE('VRDA'),
    kDTkVR_DecimalString           = FOUR_CHAR_CODE('VRDS'),
    kDTkVR_DateTime                = FOUR_CHAR_CODE('VRDT'),
    kDTkVR_FloatingPointSingle     = FOUR_CHAR_CODE('VRFL'),
    kDTkVR_FloatingPointDouble     = FOUR_CHAR_CODE('VRFD'),
    kDTkVR_IntegerString           = FOUR_CHAR_CODE('VRIS'),
    kDTkVR_LongString              = FOUR_CHAR_CODE('VRLO'),
    kDTkVR_LongText                = FOUR_CHAR_CODE('VRLT'),
    kDTkVR_OtherByteString         = FOUR_CHAR_CODE('VROB'),
    kDTkVR_OtherFloatString        = FOUR_CHAR_CODE('VROF'),
    kDTkVR_OtherWordString         = FOUR_CHAR_CODE('VROW'),
    kDTkVR_PersonName              = FOUR_CHAR_CODE('VRPN'),
    kDTkVR_ShortString             = FOUR_CHAR_CODE('VRSH'),
    kDTkVR_SignedLong              = FOUR_CHAR_CODE('VRSL'),
    kDTkVR_SequenceOfItems         = FOUR_CHAR_CODE('VRSQ'),
    kDTkVR_SignedShort             = FOUR_CHAR_CODE('VRSS'),
    kDTkVR_ShortText               = FOUR_CHAR_CODE('VRST'),
    kDTkVR_Time                    = FOUR_CHAR_CODE('VRTM'),
    kDTkVR_UniqueIdentifier        = FOUR_CHAR_CODE('VRUI'),
    kDTkVR_UnsignedLong            = FOUR_CHAR_CODE('VRUL'),
    kDTkVR_Unknown                 = FOUR_CHAR_CODE('VRUN'),
    kDTkVR_UnsignedShort           = FOUR_CHAR_CODE('VRUS'),
    kDTkVR_UnlimitedText           = FOUR_CHAR_CODE('VRUT'),
    kDTkVR_USorSS                  = FOUR_CHAR_CODE('VRU2'),
    kDTkVR_USorSSorOW              = FOUR_CHAR_CODE('VRU3'),
    kDTkVR_OWorOB                  = FOUR_CHAR_CODE('VRO2'),
    kDTkVR_SequenceItem            = FOUR_CHAR_CODE('VRS1'),
    kDTkVR_SeqItemDelimDataElement = FOUR_CHAR_CODE('VRS2'),
    kDTkVR_SeqDataElementEnd       = FOUR_CHAR_CODE('VRS3'),
    kDTkVR_NODICTENTRY             = FOUR_CHAR_CODE('VRND') 
} DTkVR;

#pragma mark enum DTk Special Value Multiplicities
/*!
	@defined kDTkVM_VariableN
    @abstract API defined constants used in conjunction with the DICOM dictionary to specify variable and undefined value multiplicity (VM) range limits<br \><br \>
    		  kDTkVM_VariableN  Indicates a VM upper limit of the form n
    @discussion The DICOM dictionary (Part 6 of DICOM standard) specifies the VM or VM range of the of the data elements whose tags it contains (i.e. public tags). 
    			Many tags have a variable VM specified as 1..n or 1..2n etc. These special VMs are represented by the constants defined in this enumeration.
                There is also a constant for an undefined multiplicity, which is the case for many retired tags that appear in the dictionary.
*/
#define kDTkVM_VariableN  ((UInt32) 0xfffffffc)

/*!
	@defined kDTkVM_Variable2N
	@abstract kDTkVM_Variable2N  Indicates a VM upper limit of the form 2n
*/
#define kDTkVM_Variable2N ((UInt32) 0xfffffffd)

/*!
	@defined kDTkVM_Variable3N
	@abstract kDTkVM_Variable3N  Indicates a VM upper limit of the form 3n
*/
#define kDTkVM_Variable3N ((UInt32) 0xfffffffe)

/*!
	@defined kDTkVM_Undefined
	@abstract kDTkVM_Undefined  Indicates an undefined VM
*/
#define kDTkVM_Undefined  ((UInt32) 0xffffffff)

#pragma mark struct DTkTagDictInfo

/*!
    @typedef DTkTagDictInfo
    @abstract Information about a data element as retrieved from the DICOM dictionary
    @field group            The group number of the data element
    @field element          The element number of the data element
    @field dictType         The value representation of the data element
    @field status           The status of the data element (active/retired)
    @field multiplicity_min The min valid value multiplicity of the data element
    @field multiplicity_max The max valid value multiplicity of the data element
    @field description      The description of the data element
    @field reserved         Reserved field for future expansion;
*/
typedef struct 
{
    UInt16            group;
    UInt16            element;
    DTkVR             dictType;
    DTkElementStatus  status;
    UInt32            multiplicity_min;
    UInt32            multiplicity_max;    
    const SInt8      *description;
    UInt32            reserved;
} DTkTagDictInfo, *DTkTagDictInfoPtr;

#pragma mark struct DTkElementDataInfo

/*!
    @typedef DTkElementDataInfo
    @abstract This structure contains information about a DICOM data element retrieved by parsing the DICOM file which contains it.
    @discussion In contrast to the DTkTagDictInfo structure, the information in DTkElementDataInfo is retrieved 
                from the actual instance of the data element in a DICOM file. As a result, possibly ambiguous information returned 
                in a DTkTagDictInfo, such as the value representation or the value mutiplicity of the data element is here resolved.
    @field numItems     The actual value multiplicity of the data element
    @field dataSize     The actual size of the data contained in the data element
    @field type         The actual value representation of the data contained in the data element
    @field offsetInFile The offset of the data element content in the file. This offset is right after the (group, element) tag of the data element
*/
typedef struct 
{
    UInt32 numItems;
    UInt32 dataSize;
    DTkVR  type;
    UInt32 offsetInFile;    
} DTkElementDataInfo, *DTkElementDataInfoPtr;

#define kDTkUndefinedLength ((UInt32) 0xffffffff)

#pragma mark -
#pragma mark enum DTkTransferSyntax
/*!
    @enum DTkTransferSyntax
    @abstract Enumeration of transfer syntaxes defined by the DICOM standard    
*/
typedef enum
{
	kDTkTransferSyntax_ImplicitVRLittleEndian,
	kDTkTransferSyntax_Default = kDTkTransferSyntax_ImplicitVRLittleEndian,
	kDTkTransferSyntax_ExplicitVRLittleEndian,
	kDTkTransferSyntax_DeflatedExplicitVRLittleEndian,
	kDTkTransferSyntax_ExplicitVRBigEndian,
	kDTkTransferSyntax_JPEGBaseline_Process1,
	kDTkTransferSyntax_JPEGExtended_Process2and4,
	kDTkTransferSyntax_JPEGExtended_Process3and5,
	kDTkTransferSyntax_JPEGSpectralSelectionNonHierarchical_Process6and8,
	kDTkTransferSyntax_JPEGSpectralSelectionNon_Hierarchical_Process7and9,
	kDTkTransferSyntax_JPEGFullProgressionNonHierarchical_Process10and12,
	kDTkTransferSyntax_JPEGFullProgressionNonHierarchical_Process11and13,
	kDTkTransferSyntax_JPEGLosslessNonHierarchical_Process14,
	kDTkTransferSyntax_JPEGLosslessNonHierarchical_Process15,
	kDTkTransferSyntax_JPEGExtendedHierarchical_Process16and18,
	kDTkTransferSyntax_JPEGExtendedHierarchical_Process17and19,
	kDTkTransferSyntax_JPEGSpectralSelectionHierarchical_Process20and22,
	kDTkTransferSyntax_JPEGSpectralSelectionHierarchical_Process21and23,
	kDTkTransferSyntax_JPEGFullProgressionHierarchical_Process24and26,
	kDTkTransferSyntax_JPEGFullProgressionHierarchical_Process25and27,
	kDTkTransferSyntax_JPEGLosslessHierarchical_Process28,
	kDTkTransferSyntax_JPEGLosslessHierarchical_Process29,
	kDTkTransferSyntax_JPEGLosslessNonHierarchical_First_OrderPrediction_Process14,
	kDTkTransferSyntax_JPEG_LSLosslessImageCompression,
	kDTkTransferSyntax_JPEG_LSLossy_Near_LosslessImageCompression,
	
	kDTkTransferSyntax_JPEG2000ImageCompression_LosslessOnly,
	kDTkTransferSyntax_JPEG2000ImageCompression,
	kDTkTransferSyntax_JPEG2000_MultipleComponent_ImageCompression_LosslessOnly,
	kDTkTransferSyntax_JPEG2000_MultipleComponent_ImageCompression,
	
	kDTkTransferSyntax_RLELossless,
	
	kDTkTransferSyntax_Unknown = kDTkUnknown 
} DTkTransferSyntax;

#pragma mark enum DTkEndianness

/*!
    @enum DTkEndianness
    @abstract Type definition of enumeration of allowed values for data endianness of multi-byte values.
    @constant kDTkLittleEndian Signifies a little endian encoding
    @constant kDTkBigEndian    Signifies big endian encoding
    @constant kDTkNoEndian     Endian does not apply to encoding
    @constant kDTkUnknown      Identifies an encoding of unknown endian
*/
typedef enum
{
    kDTkLittleEndian,
    kDTkBigEndian,
    kDTkNoEndian,
    kDTkUnkownEndian = kDTkUnknown
} DTkEndianness;


#pragma mark enum DTkPixelRepresentation

/*!
    @enum DTkPixelRepresentation
    @abstract Enumeration of DICOM Pixel Representations.
    @constant kDTkUnsignedInteger This value is equal to the value used by the DICOM standard to specify the
                              Unsigned Integer pixel representation
    @constant kDTkTwosComplement This value is equal to the value used by the DICOM standard to specify the
                              Two's complement pixel representation
    @constant kDTkUnknownPixelRepresentation This is the value used to initialize variables of type DTkDataPixelRepresentation
*/
typedef enum
{
    kDTkUnsignedInteger		= 0,
    kDTkTwosComplement		= 1,
    kDTkUnknownPixelRepresentation = kDTkUnknown
} DTkPixelRepresentation;



#pragma mark enum DTkVRVerbosity

/*!
    @enum DTkVRVerbosity
    @abstract Enumeration of DICOM value representation verbosity.
    @constant kDTkImplicitVR DICOM data elements have an implicit value representation
    @constant kDTkExplicitVR DICOM data elements have an explicit value representation
    @constant kDTkUnknownVR  DICOM data elements have an unknown value representation
*/
typedef enum
{
    kDTkImplicitVR,
    kDTkExplicitVR,
    kDTkUnknownVR = kDTkUnknown
} DTkVRVerbosity;


#pragma mark enum DTkBasicOffsetTableType

/*!
	@enum DTkBasicOffsetTableType
	@abstract Enumeration of basic offset table possibilities.
	@discussion One of the enumerated values will be returned when DTkParseGetFileMetaInformation is passed the kDTkBasicOffsetTableTypeSelector
    @constant kDTkBasicOffsetTableType_ExistsFull   The DICOM file contains a full basic offset table
    @constant kDTkBasicOffsetTableType_ExistsEmpty  The DICOM file contains an empty basic offset table 
    @constant kDTkBasicOffsetTableType_DoesNotExist The DICOM file does not contain a basic offset table 
    @constant kDTkBasicOffsetTableType_Unknown      Signifies an unknown offset table type
*/
typedef enum
{
	kDTkBasicOffsetTableType_ExistsFull,
	kDTkBasicOffsetTableType_ExistsEmpty,
	kDTkBasicOffsetTableType_DoesNotExist,
	kDTkBasicOffsetTableType_Unknown = kDTkUnknown
} DTkBasicOffsetTableType;

#pragma mark DTkPixelDataLengthType

/*!
	@enum DTkPixelDataLengthType
	@abstract Enumeration of pixel data (0x7FE0, 0x0010) element size possibilities.
	@discussion One of the enumerated values will be returned when DTkParseGetFileMetaInformation is passed the kDTkBasicOffsetTableTypeSelector
    @constant kDTkPixelDataLengthType_Explicit   The pixel data element has an explicit length specified
    @constant kDTkPixelDataLengthType_Undefined  The pixel data element length is undefined (i.e. set to 0xFFFFFFFF)
    @constant kDTkPixelDataLengthType_Unknown    Signifies an unknown pixel data element length type
*/
typedef enum
{
	kDTkPixelDataLengthType_Explicit,
	kDTkPixelDataLengthType_Undefined,
	kDTkPixelDataLengthType_Unknown = kDTkUnknown
} DTkPixelDataLengthType;


#pragma mark enum DTkMetaInformationSelector

/*!
    @enum DTkMetaInformationSelector
    @abstract Enumeration of the allowed values for the DTkMetaInformationSelector type parameter of the 
              DTkParseGetFileMetaInformation API function.
    @discussion The vales returned refer to those that are specified in the DICOM file.
    @constant kDTkEndiannessSelector           Instructs DTkParseGetFileMetaInformation to return the endianness of the DICOM data set
    @constant kDTkPixelRepresentationSelector  Instructs DTkParseGetFileMetaInformation to return the pixel representation of the pixel data
    @constant kDTkDataVRVerbositySelector      Instructs DTkParseGetFileMetaInformation to return the verbosity type of the DICOM data set
    @constant kDTkDataTransferSyntaxSelector   Instructs DTkParseGetFileMetaInformation to return the transfer syntax of the DICOM data set
    @constant kDTkBasicOffsetTableTypeSelector Instructs DTkParseGetFileMetaInformation to return the basic offset table type of the DICOM data set
*/
typedef enum
{
    kDTkEndiannessSelector          = FOUR_CHAR_CODE('S_EN'),
    kDTkPixelRepresentationSelector = FOUR_CHAR_CODE('S_PR'),
    kDTkDataVRVerbositySelector     = FOUR_CHAR_CODE('S_VR'),
    kDTkDataTransferSyntaxSelector  = FOUR_CHAR_CODE('S_TS'),   
    kDTkBasicOffsetTableTypeSelector= FOUR_CHAR_CODE('S_OT'),
    kDTkPixelDataLengthTypeSelector = FOUR_CHAR_CODE('S_PD')
} DTkMetaInformationSelector;

#pragma mark -
#pragma mark struct DTkPixelRepresentationsRecord

/*!
    @typedef DTkPixelRepresentationsRecord
    @abstract Structure used to return the value of the pixel representation during various stages of the DICOM Image manipulation pipeline.
    @field original                    Pixel representation of DICOM image as specified in the DICOM data set.
    @field afterModalityTransformation Pixel representation of DICOM image after a possible modality transformation was applied to the image data.
    @field final                       Pixel representation of DICOM image after all transformations have been applied to the image data.
*/
typedef struct
{
    DTkPixelRepresentation original;
    DTkPixelRepresentation afterModalityTransformation;
    DTkPixelRepresentation final;
} DTkPixelRepresentationsRecord, *DTkPixelRepresentationsRecordPtr;


#pragma mark enum DTkPixelInversionStatus

/*!
    @enum DTkPixelInversionStatus
    @abstract Constants used to identify whether pixels have been inverted during decompression.
    @constant kDTkPixelsWereInverted This value signifies that the colors of the image have been inverted during decompression of the image data.
    @constant kDTkPixelsWereNotInverted This value signifies that the colors of the image have not been inverted during decompression of the image data.
    @constant kDTkUnknownPixelInversionStatus Value used to initialize variables of type DTkPixelInversionStatus.
*/
typedef enum
{
    kDTkPixelsWereInverted,
    kDTkPixelsWereNotInverted,
    kDTkUnknownPixelInversionStatus = kDTkUnknown
} DTkPixelInversionStatus;


#pragma mark struct DTkFileVOIParams

/*!
    @typedef DTkFileVOIParams
    @abstract Structure that holds VOI related data for an image.
    @field VOICoefsExistInFile Flag that specifies if VOI parameters exist in a file.
    @field fileWindowCenter    First value of VOI Window Center that was specified in the DICOM file.
    @field fileWindowWidth     First value of VOI Window Width that was specified in the DICOM file.
*/
typedef struct 
{
    Boolean VOICoefsExistInFile;
    UInt8   pad[3];
    Float32 fileWindowCenter;
    Float32 fileWindowWidth;
} DTkFileVOIParams, *DTkFileVOIParamsPtr;

#pragma mark enum DTkPhotometricInterpretation

/*!
    @enum DTkPhotometricInterpretation
    @abstract Enumaration of the various photometric interpretations defined by the DICOM standard.
    @discussion The enumerated constants are the allowed values for the photoInterpretation field of the DTkPixelInfo structure.
*/
typedef enum
{
    kDTkMONOCHROME1,
    kDTkMONOCHROME2,
    kDTkPALETTE_COLOR,
    kDTkRGB,
    kDTkHSV,
    kDTkARGB,
    kDTkCMYK,
    kDTkYBR_FULL,
    kDTkYBR_FULL_422,
    kDTkYBR_PARTIAL_422,
    kDTkYBR_ICT,
    kDTkYBR_RCT,
    kDTkUnknownPhotometricInterpretation = kDTkUnknown
} DTkPhotometricInterpretation;


#pragma mark struct DTkMinMaxPixelValues
/*!
    @typedef DTkMinMaxPixelValues
    @abstract Structure containing the minimum and maximum pixel values of the image data 
    (after any modality and before any VOI transformation)
    @discussion The actual values correspond to the actual pixel data and may change from 
    			frame to frame in a series or a multiimag file.
			    The nominal value are calculated from various Image Pixel Module attributes.
			    The latter values are invariant across multiple frames and are bounds for
			    the actual pixel minimum and maxium values of the pixel data.
    @field minPixelValueActual The actaul minimum pixel value of the image data
    @field maxPixelValueActual The actaul maximum pixel value of the image data
    @field minPixelValueNominal The nomial minimum pixel value of the image data
    @field maxPixelValueNominal The nomial maximum pixel value of the image data
*/
typedef struct
{
    SInt32 minPixelValueActual;
    SInt32 maxPixelValueActual;
    SInt32 minPixelValueNominal;
    SInt32 maxPixelValueNominal;
} DTkMinMaxPixelValues, *DTkMinMaxPixelValuesPtr,  **DTkMinMaxPixelValuesHdl;

typedef SInt32 DTkPixelRangeShiftValueType;
typedef SInt32 DTkPixelValueReversalFactorType;

#pragma mark enum DTkPixelInfoSelector
/*!
    @enum DTkPixelInfoSelector
    @abstract Enumeration of the allowed values for the DTkPixelInfoSelector type parameter of the 
              DTkCodecGetPixelInfo API function.
    @constant kDTkPixelRepresentationsRecordSelector Instructs DTkCodecGetPixelInfo to return a DTkPixelRepresentationsRecord structure
    @constant kDTkPixelInversionStatusSelector       Instructs DTkCodecGetPixelInfo to return the pixel inversion status
    @constant kDTkFileVOIParamsSelector              Instructs DTkCodecGetPixelInfo to return a DTkFileVOIParams structure
    @constant kDTkPhotometricInterpretationSelector  Instructs DTkCodecGetPixelInfo to return the photometric interpretation of the image data
    @constant kDTkMinMaxPixelValuesSelector          Instructs DTkCodecGetPixelInfo to return a DTkMinMaxPixelValues structure
    @constant kDTkPixelRangeShiftValueSelector		 Instructs DTkCodecGetPixelInfo to return the value of type DTkPixelRangeShiftValueType
    												 by which the pixels have been shifted in order to be brought into the positive range
    @constant kDTkPixelValueReversalFactorSelector   Instructs DTkCodecGetPixelInfo to return the value of type DTkPixelValueReversalFactorType
    												 out of which the pixels have been subtracted during a pixel value reversal opreation 
    												 triggered by their photometric interpretation
        
*/
typedef enum
{
    kDTkPixelRepresentationsRecordSelector = FOUR_CHAR_CODE('S_PX'),
    kDTkPixelInversionStatusSelector       = FOUR_CHAR_CODE('S_IN'),
    kDTkFileVOIParamsSelector              = FOUR_CHAR_CODE('S_VP'),
    kDTkPhotometricInterpretationSelector  = FOUR_CHAR_CODE('S_PI'),
    kDTkMinMaxPixelValuesSelector          = FOUR_CHAR_CODE('S_MM'),
    kDTkPixelRangeShiftValueSelector	   = FOUR_CHAR_CODE('S_PS'),
    kDTkPixelValueReversalFactorSelector   = FOUR_CHAR_CODE('S_PR')	
} DTkPixelInfoSelector;

#pragma mark -
#pragma mark enum DTkCodecHints

/*!
    @enum DTkCodecHints
    @abstract Flags to be used in conjunction with API function DTkCodecSetHints and the
              DTkDecompressorHintsImageDescExtension (Decompressor Hints Image Dedcription Extension).
    @discussion These flags are instructions to the DICOM decompressor and should be 
                applied before a particular frame is decompressed. They control internal 
                actions of the decompressor. 
    @constant kDTkCodecApplyVOITransform            Instructs the codec to apply the VOI (values of interest) transformation.
                                                        Not to be set when dynamic application of the VOI transformation is required.
    @constant kDTkCodecApplyRescaleTransform        Instructs the codec to apply the rescale (slope/intercept) transformation.
                                                        This should usually be set to allow for application of hardware dependent
                                                        pixel value corrections.
    @constant kDTkCodecOccupyFullRange              Instructs the codec to scale pixels to use entire available value range
                                                        This hint s obsolete and is ignored.
    @constant kDTkCodecGetPixelRangeFromData        Instructs the codec to calculate and use the min/max values obtained by parsing the image data
                                                        ignoring the min/max pixel values given in the file. Although is may impose a small speed 
                                                        penalty, it is saves one from using wrong values that are occasionally found in DICOM files.
                                                        If there are no such data elements in a file, they are calculated anyway.
    @constant kDTkCodecRespectSignedData            Instructs the codec <b>not</b> to make signed pixels unsigned (by adding a
                                                        constant equal to half the maximum possible numeric range of the data). 
                                                        Effects images with signed data (after the application of any Slope/Intercept
                                                        transformations). The decompressor's VOI commands work under the implicit assumption that 
                                                        this flag is not set.
    @constant kDTkCodecGetFinalMinMaxPixelValues    Instructs the codec to calculate the min/max pixel values of the final pixel
                                                        data (after all transformations). To be used with dynamic VOI transformations.
	@constant kDTkCodecApplyMaskDuringDecompression Instructs the codec to perform a mask operation during decompression of the image.
	                                                    If during decompression no mask data are found an error is emitted.		
    @constant kDTkCodecApplyVOILUTDuringDecompression Instructs the codec to apply the VOI (values of interest) transformation using a VOI
                                                        lookup table if one exists in the file.
*/
typedef enum {
    kDTkCodecApplyVOITransform             = (1L << 0),
    kDTkCodecApplyRescaleTransform         = (1L << 1),
    kDTkCodecOccupyFullRange               = (1L << 2),
    kDTkCodecGetPixelRangeFromData         = (1L << 3),
    kDTkCodecRespectSignedData             = (1L << 4),
    kDTkCodecGetFinalMinMaxPixelValues     = (1L << 5),
    kDTkCodecApplyMaskDuringDecompression  = (1L << 6),
    kDTkCodecApplyVOILUTDuringDecompression= (1L << 7)
//  kDTkCodecIsMaskFrame				   = (1L << 8)
} DTkCodecHints;

#pragma mark enum DTkImageDescExtensionTypes

/*!
    @enum DTkImageDescExtensionTypes
    @abstract Four character codes identifying the image description extensions (IDEs) used by DICOM Toolkit
    @constant kDTkImgDescExtVOIParamsType Four character code identifying an IDE structure as a 
                                               DTkVOIParamsImageDescExtension
    @constant kDTkImgDescExtDecompressorHints Four character code identifying an IDE structure as a 
                                               DTkDecompressorHintsImageDescExtension
    @constant kDTkImgDescExtVOIBoundsClippingOption Four character code identifying an IDE structure as a 
                                               DTkVOIBoundsClippingOptionImageDescExtension
    @constant kDTkImgDescExtDisplayPixelBounds Four character code identifying an IDE structure as a 
                                               DTkDisplayPixelBoundsImageDescExtension
    @constant kDTkImgDescExtVOIParamsArrayType Four character code identifying an IDE structure as a 
    										   DTkVOIParamPairArrayImageDescExtension
	@constant kDTkImageDescriptionColorSpace Specifies an image description extension type analogous to QuickTime's
                                             Color Space image description extension to be used with color spaces 
                                             private to DTk. The image description extension contains an OSType
                                             naming the native pixel format of an image 
                                             (only used for pixel formats not supported by QuickTime)
    			
*/
typedef enum
{
    kDTkImgDescExtVOIParamsType           = FOUR_CHAR_CODE('DVOI'),
	kDTkImgDescExtDecompressorHints       = FOUR_CHAR_CODE('DHNT'),
    kDTkImgDescExtVOIBoundsClippingOption = FOUR_CHAR_CODE('DVCL'),
    kDTkImgDescExtDisplayPixelBounds      = FOUR_CHAR_CODE('DDPB'),
	kDTkImgDescExtVOIParamsArrayType      = FOUR_CHAR_CODE('DVPA'),
	kDTkImgDescExtColorSpace              = FOUR_CHAR_CODE('DCSP')
} DTkImageDescExtensionTypes;


#pragma mark struct DTkColorSpaceImageDescExtension
/*!
    @typedef DTkColorSpaceImageDescExtension
    @abstract A structure containing the image description extension corresponding to the kImageDescriptionColorSpace type
    @field pixelFormat OSType naming the native pixel format of an image
*/
typedef struct  {
    UInt32 pixelFormat;
} DTkColorSpaceImageDescExtension;
typedef DTkColorSpaceImageDescExtension *DTkColorSpaceImageDescExtensionPtr;
typedef DTkColorSpaceImageDescExtensionPtr *DTkColorSpaceImageDescExtensionHdl;

#pragma mark struct DTkVOIParamsRec

/*!
    @typedef DTkVOIParamsRec
    @abstract Structure containing a set VOI parameters in the form used
              with DTK's VOI Blitter calls, i.e. as integers.
    @field windowCenter The window center
    @field windowWidth  The window width
*/
typedef struct
{
    SInt32 windowCenter;
    UInt32 windowWidth;
} DTkVOIParamsRec;

/*!
    @typedef DTkVOIParamsRec
    @abstract VOI parameters expresed as start and end values of the VOI window
    @field windowStart The starting value of the VOI window
    @field windowEnd   The ending value of the VOI window
*/
typedef struct
{
	SInt32 windowStart;
	SInt32 windowEnd;
} DTkVOIBoundsRec, *DTkVOIBoundsRecPtr;


typedef DTkVOIParamsRec DTkVOIParamsImageDescExtension;
typedef DTkVOIParamsImageDescExtension    *DTkVOIParamsImageDescExtensionPtr;
typedef DTkVOIParamsImageDescExtensionPtr *DTkVOIParamsImageDescExtensionHdl;

/*!
    @typedef DTkDecompressorHintsImageDescExtension
    @abstract Structure containing the contents of a VOI Decompressor Hints Image Description Extension
    @field dicomDecompressorHints The Decompressor Hints
*/
typedef struct 
{
	UInt32	dicomDecompressorHints;
} DTkDecompressorHintsImageDescExtension;
typedef DTkDecompressorHintsImageDescExtension      *DTkDecompressorHintsImageDescExtensionPtr;
typedef DTkDecompressorHintsImageDescExtensionPtr	*DTkDecompressorHintsImageDescExtensionHdl;

#pragma mark enum DTkVOIBoundsClippingOption
/*!
    @enum DTkVOIBoundsClippingOption
    @abstract This enumeration is used by the DTkCodecSetVOIBoundsClippingOption to control the clipping performed 
              by the toolkit's DTkCodecVOICopyPixels16To8bits, DTkCodecVOICopyPixels8To8bits and DTkCodecGetPixelValue functions.
    @constant kDTkDontOptimizeVOIBounds When this flag is set no clipping of the VOI bounds occurs. This is the cleanest approach, 
                                        in that the VOI transformation strictly adheres to the currently active window width and window center. 
    @constant kDTkClipVOIBoundsToActualPixelBounds  When this flag is set the VOI bounds are clipped to the 
                                                        actual pixel bounds (as extracted from the image data). This option results in the 
                                                        best possible contrast for the image.
    @constant kDTkClipVOIBoundsToNominalPixelBounds When this flag is set the VOI bounds are clipped to the 
                                                        nominal pixel bounds (as calculated from the Bits Stored (0x0028,0x0101))
                                                        attribute of the DICOM file.
    @constant kDTkClipVOIBoundsToUserPixelBounds When this flag is set the VOI bounds are clipped to the 
                                                     pixel bounds that have been set by the user of the toolkit 
                                                     (using the DtkCodecSetDisplayPixelBounds call).
    @constant kDTkUnknownVOIBoundsClippingOption Specifies an unknown VOI bounds clipping option.
    
*/
typedef enum
{
    kDTkDontOptimizeVOIBounds,
    kDTkClipVOIBoundsToActualPixelBounds,
    kDTkClipVOIBoundsToNominalPixelBounds,
    kDTkClipVOIBoundsToUserDefinedPixelBounds,
    kDTkUnknownVOIBoundsClippingOption = kDTkUnknown
} DTkVOIBoundsClippingOption;


#pragma mark struct DTkVOIBoundsClippingOptionImageDescExtension

/*!
    @typedef DTkVOIBoundsClippingOptionImageDescExtension
    @abstract Structure containing the contents of a VOI Bounds Clipping Option Image Description Extension
    @field clippingOption The clipping option to be used when applying a VOI transformation
*/
typedef struct
{
    DTkVOIBoundsClippingOption clippingOption;
} DTkVOIBoundsClippingOptionImageDescExtension;
typedef DTkVOIBoundsClippingOptionImageDescExtension *DTkVOIBoundsClippingOptionImageDescExtensionPtr;
typedef DTkVOIBoundsClippingOptionImageDescExtensionPtr *DTkVOIBoundsClippingOptionImageDescExtensionHdl;


#pragma mark struct DTkDisplayPixelBoundsImageDescExtension

/*!
    @typedef DTkDisplayPixelBoundsImageDescExtension
    @abstract This Image Description Extension specifies the minimum and maximum pixel 
              values that will define the range to be mapped to the output range during a 
              VOI transformation. 
    @discussion These parameters are useful for maintaining an optimal source pixel range across multiple frames.
               See also the documentation of the DTkCodecSetVOIBoundsClippingOption call.
    @field displayMinPixelValue The minimum pixel value
    @field displayMaxPixelValue The maximum pixel value
*/
typedef struct
{
     SInt32 displayMinPixelValue;
     SInt32 displayMaxPixelValue;
    
} DTkDisplayPixelBoundsImageDescExtension;
typedef DTkDisplayPixelBoundsImageDescExtension *DTkDisplayPixelBoundsImageDescExtensionPtr;
typedef DTkDisplayPixelBoundsImageDescExtensionPtr *DTkDisplayPixelBoundsImageDescExtensionHdl;

typedef ComponentInstance DTkParserComponentInstance;

/*!
    @typedef DTkVOIParamPair
    @abstract Structure containing a set VOI parameters, in the form used with a 
              DICOM data set, i.e. as floating point numbers.
    @field windowCenter The window center
    @field windowWidth  The window width
*/
typedef struct
{
    Float32 windowCenter;
    Float32 windowWidth;
} DTkVOIParamPair;

/*!
    @typedef DTkVOIParamPairArrayImageDescExtension
    @abstract Structure containing an array of DTkVOIParamPair structs (VOI parameter pairs).
    @field numElements The number of elements iof the array
    @field VOIparams   The array of DTkVOIParamPair structs
*/

typedef struct
{
    UInt32 numElements;
    DTkVOIParamPair VOIparams[1];
} DTkVOIParamPairArrayImageDescExtension;
typedef DTkVOIParamPairArrayImageDescExtension *DTkVOIParamPairArrayImageDescExtensionPtr;
typedef DTkVOIParamPairArrayImageDescExtensionPtr *DTkVOIParamPairArrayImageDescExtensionHandle;


#pragma mark -
#pragma mark enum DTkParserFourCharCodes

/*!
    @enum DTkParserFourCharCodes    
    @abstract DICOM parser component identification constants
    @discussion These constants are useful when creating an instance of a parser component e.g. to populate the ComponentDescription
    argument of QuickTime's    OpenAComponent function.
    @constant kDTkParserComponentType         Four character code of DICOM Parser component type
    @constant kDTkParserComponentSubType      Four character code of DICOM Parser component sub-type
    @constant kDTkParserComponentManufacturer Four character code of DICOM Parser component manufacturer
*/
typedef enum 
{
    kDTkParserComponentType         = FOUR_CHAR_CODE('PRSR'),
    kDTkParserComponentSubType      = FOUR_CHAR_CODE('DICM'),
    kDTkParserComponentManufacturer = FOUR_CHAR_CODE('ESC ')
} DTkParserFourCharCodes;

#pragma mark enum DTkStandardFlags

/*!
    @enum DTkStandardFlags
    @abstract Definitions of constants to be passed as flags to control the operation of the DICOM Parser component via the function DTkParseSetParserFlags.
    @constant kDTkStandardFlagIgnoreSpecialSequenceVRs This flag is to be used with the DTkStandardGetDictInfoForTag and instructs it
              to ignore sequence value representations that are not defined by the DICOM standard.  
              When the flag is set, the value representation of elements of API defined types kDTkVR_SequenceItem, 
              kDTkVR_SeqItemDelimDataElement, kDTkVR_SeqDataElementEnd is set to kDTkVR_Empty, 
              strictly adhering to the standard.
              
*/
typedef enum
{
    kDTkStandardFlagIgnoreSpecialSequenceVRs = (1L << 0)
} DTkStandardFlags;


#pragma mark enum DTkParserFlags

/*!
    @enum DTkParserFlags
    @abstract Definitions of constants to be passed as flags to control the operation of the DICOM Parser component via the function DTkParseSetParserFlags.
    @constant kDTkParserFlagIgnoreSpecialSequenceVRs This flag instructs the DICOM parser to ignore sequence value representations that are not defined by 
              the DICOM standard.  Affects the output of functions DTkParseGetElementDataInfo and DTkParseGetDictInfoForTag. 
              When the flag is set, the value representation of elements of API defined types kDTkVR_SequenceItem, 
              kDTkVR_SeqItemDelimDataElement, kDTkVR_SeqDataElementEnd is set to kDTkVR_Empty, 
              strictly adhering to the standard.
*/
typedef enum
{
    kDTkParserFlagIgnoreSpecialSequenceVRs = kDTkStandardFlagIgnoreSpecialSequenceVRs,
    kDTkParserFlagDontEndianFlipRawData	   = (1L << 1)
} DTkParserFlags;


/*!
    @enum DTkParseGetNumberOfFramesFlags
    @abstract Flags modifying the behavior of the DTkParseGetNumberOfFrames call
    @constant kGetNumberOfFramesFlag_GetNominalNumber When the flag is set the nominal number of frames in the file is
                									  returned by DTkParseGetNumberOfFramesFlags. When it is not set (that is, it is set to 
                									  kGetNumberOfFramesFlag_GetActualNumber) the actual number of frames in the file is returned. 
                									  The two retrieved values may differ with DICOM files that are incomplete, 
                									  e.g. because of a file transfer error.
*/
typedef enum
{
	kGetNumberOfFramesFlag_GetActualNumber	= 0,
	kGetNumberOfFramesFlag_GetNominalNumber = 1
} DTkParseGetNumberOfFramesFlags;

#pragma mark enum DTkValidationFlags

/*!
    @enum DTkValidationFlags
    @discussion
        Currently only the kDTkValidationFast flag is supported. With is flag enabled DTkParseValidateDataRef does not
        perform an exhaustive check on the file.
        It is possible for DTkParseValidateDataRef to claim a data reference is valid but for 
        other parser commands to return an error due to bad data. However, it is very fast and thus suitable
        for use in situations where one wishes identify DICOM files among collections of large numbers of arbitrary files.
    @abstract Validation flags which are to be used with DTkParseValidateDataRef control the thoroughness of the validation performed.
    @constant kDTkValidationFast Quick validation that only checks if file has DICOM formatted data.
*/
typedef enum
{
    kDTkValidationFast = 0
} DTkValidateDataRefFlags;


#pragma mark enum DTkUserToDisplayConversionFlags

/*!
    @enum DTkUserToDisplayConversionFlags
    @abstract Flags to be used with DTkCodecDisplayToUserValue
    @constant kDTkDontCheckBounds When this flag is set DTkCodecDisplayToUserValue does not perform 
                                  a bounds check on the inputted display value.
*/
typedef enum
{
    kDTkDontCheckBounds = (1L << 0)
} DTkUserToDisplayConversionFlags;


/*!
    @enum DTkUserDefaultVOIParamsSelector
    @abstract The constants defined in this enumeration are to be used with the DTkCodecGetUserDefaultVOIParams function.
    @constant kDTkGetFirstVOIParamSetFromFileFallBackToOptimal	Specifies that if VOI parameters exist in the file, 
         the first set is retrieved. If no VOI parameters are found in the file, optimal values are calculated from the 
	     current image such that the after their application the image occupies the full available 8 bit output range. 
    @constant kDTkGetVOIParamsForOptimalTransformation  Instructs DTkCodecGetUserDefaultVOIParams to return the optimal VOI parameters for the image. 
    @constant kDTkGetVOIParamsForIdentityTransformation Instructs DTkCodecGetUserDefaultVOIParams to return the VOI parameters that correspond to an identity VOI transformation
    @constant kDTkGetVOIParamsForUserPixelBounds   Instructs DTkCodecGetUserDefaultVOIParams to return the VOI parameters 
         that correspond the pixel bounds set by a previous invocation of the DTkCodecSetDisplayPixelBounds call.  
*/
typedef enum
{
    kDTkGetFirstVOIParamSetFromFileFallBackToOptimal =  0,
    kDTkGetVOIParamsForOptimalTransformation         = -1,
    kDTkGetVOIParamsForIdentityTransformation        = -2,
    kDTkGetVOIParamsForUserPixelBounds               = -3
} DTkUserDefaultVOIParamsSelector;



/*!
    @enum DTkRepeatingGroupBounds
    @abstract The constants defined in this enumeration delimit the allowed ranges 
              repeating group defined by the DICOM standard.
    @constant kDTkFirstCurveRepeatingGroup The minimum group number a curve repeating group may have
    @constant kDTkLastCurveRepeatingGroup The maximum group number a curve repeating group may have
    @constant kDTkFirstOverlayRepeatingGroup The minimum group number an overlay repeating group may have
    @constant kDTkLastOverlayRepeatingGroup The maximum group number an overlay repeating group may have
	@discussion In Part 5, Section 7.6 the DICOM standard specifies the following:
                Multiple Overlay Planes and Curves are often associated with a single Image. Standard Data
                Elements with even Group Numbers (5000-501E,eeee) represent Curves, while elements with even
                Group Numbers (6000-601E,eeee) represent Overlay Planes. Both of these ranges of Group numbers are
                known as Repeating Groups. This use of group numbers is a remnant of versions of this standard prior to
                V3.0 that associated a semantic meaning with particular Groups.
                In each of these ranges of Group Numbers, Standard Data Elements that have identical Element
                Numbers have the same meaning within each Group (and the same VR, VM, and Data Element Type).
                The notation (50xx,eeee) and (60xx,eeee) are used for the Group Number in Data Element Tags when
                referring to a common Data Element across these groups (see PS 3.6). Groups (50xx,eeee) and
                (60xx,eeee) are called Repeating Groups because of these characteristics.
                Repeating Groups shall only be allowed in the even Groups (6000-601E,eeee) and even Groups (5000-
                501E,eeee) cases. In the future, Data Elements with VRs of SQ shall be used to serve a similar purpose.
*/
typedef enum
{
	kDTkFirstCurveRepeatingGroup	= 0x5000,
	kDTkLastCurveRepeatingGroup		= 0x501E,
	kDTkFirstOverlayRepeatingGroup	= 0x6000,
	kDTkLastOverlayRepeatingGroup	= 0x601E
} DTkRepeatingGroupBounds;

/*!
	@defined kDTkParserRootElementRef
	@abstract kDTkParserRootElementRef Specfies that parsing is to take place at the top level of the DICOM hierarchy 
*/
#define kDTkParserRootElementRef (0)

/*!
    @enum DTkCompressionType
    @abstract Enumeration of supported DICOM compression types
    @constant kDTkNoCompression		Used for uncompressed image data
    @constant kDTkJPEGLossyCompression	Used for JPEG lossy compressed image data	
    @constant kDTkJPEGLosslessCompression	Used for JPEG lossless compressed image data	
    @constant kDTkJPEGLosslessCompressionProc14 Used for JPEG lossless compressed image data according to process 14
    @constant kDTkRLECompression Used for RLE (lossless) compressed image data	
    @constant kDTkUnknownCompression Denotes an unsupported compression format
*/			 
typedef enum
{
	kDTkNoCompression					= FOUR_CHAR_CODE('NONE'),
	kDTkJPEGLossyCompression			= FOUR_CHAR_CODE('JLSY'),
	kDTkJPEGLosslessCompression			= FOUR_CHAR_CODE('JLLS'),
	kDTkJPEGLosslessCompressionProc14	= FOUR_CHAR_CODE('JP14'),
	kDTkRLECompression					= FOUR_CHAR_CODE('JRLE'),
	kDTkJPEG2KLossyCompression			= FOUR_CHAR_CODE('J2LS'),
	kDTkJPEG2KLosslessCompression		= FOUR_CHAR_CODE('J2LL'),
	kDTkUnknownCompression				= kDTkUnknown
} DTkCompressionType;


/*!
    @enum DTkGraphicsImportImageDataInfoSelector
	@abstract Enumeration of the allowed values for the selector parameter of the DTkGraphicsImportGetImageDataInfo API function.
	@constant kDTkSelector_CompressionType  Instructs DTkGraphicsImportGetImageDataInfo to return the comprsession type of the DICOM image data.
	@constant kDTkSelector_HasOverlaysQuery Instructs DTkGraphicsImportGetImageDataInfo to return a Boolean vau indicating whether the DICOM file contains overlay data.
*/

typedef enum
{
	kDTkSelector_CompressionType  = FOUR_CHAR_CODE('G_CT'),
	kDTkSelector_HasOverlaysQuery = FOUR_CHAR_CODE('G_HO')
} DTkGraphicsImportImageDataInfoSelector;

#pragma mark -
#pragma mark  Support for DTkStandardFindElemInVector 
#pragma mark enum DTkElemRecTypeSelector
/*!
    @enum DTkElemRecTypeSelector
	@abstract Enumeration of the allowed values for the selector parameter of the DTkStandardFindElemInVector API function.
	@constant kDTkElemRecTypeSel_UIDRegistryElem      Specifies that the pointer passed to DTkStandardFindElemInVector is the address of a DTkUIDInfoRecElem structure.
	@constant kDTkElemRecTypeSel_StandardSOPClassElem Specifies that the pointer passed to DTkStandardFindElemInVector is the address of a DTkStandardSOPClassElem structure.
	@constant kDTkElemRecTypeSel_Unknown              Specifies an unknown record type.
*/
typedef enum
{
	kDTkElemRecTypeSel_UIDRegistryElem,
	kDTkElemRecTypeSel_StandardSOPClassElem,
	kDTkElemRecTypeSel_Unknown				 = kDTkUnknown
} DTkElemRecTypeSelector;

#pragma mark enum DTkUIDType
/*!
    @enum DTkUIDType
	@abstract Enumeration of the types UIDs defined by the standard in the UID regsistry at PS 3.6
	@constant kDTkUIDType_None                           Specifies a UID of type "None"
	@constant kDTkUIDType_SOPClass                       Specifies a UID of type "SOP Class"
	@constant kDTkUIDType_MetaSOPClass                   Specifies a UID of type "Meta SOP Class"
	@constant kDTkUIDType_TransferSyntax                 Specifies a UID of type "Transfer Syntax"
	@constant kDTkUIDType_ApplicationContextName         Specifies a UID of type "Application Contex tName"
	@constant kDTkUIDType_WellknownSOPInstance           Specifies a UID of type "Well-known SOP Instance"
	@constant kDTkUIDType_CodingScheme                   Specifies a UID of type "Coding Scheme"
	@constant kDTkUIDType_WellknownPrinterSOPInstance    Specifies a UID of type "Well-known Printer SOP Instance"
	@constant kDTkUIDType_WellknownPrintQueueSOPInstance Specifies a UID of type "Well-known Print Queue SOP Instance"
	@constant kDTkUIDType_Unknown						  Specifies an unknown UID type
*/
typedef enum
{
	kDTkUIDType_None,
	kDTkUIDType_SOPClass,
	kDTkUIDType_MetaSOPClass,
	kDTkUIDType_TransferSyntax,
	kDTkUIDType_ApplicationContextName,
	kDTkUIDType_WellknownSOPInstance,
	kDTkUIDType_CodingScheme,
	kDTkUIDType_WellknownPrinterSOPInstance,
	kDTkUIDType_WellknownPrintQueueSOPInstance,
	kDTkUIDType_Unknown = kDTkUnknown
} DTkUIDType;

#pragma mark struct DTkUIDInfoRecElem
/*!
    @typedef DTkUIDInfoRecElem
	@abstract Type definition of a structure containing information from the DICOM dictionary, as defined n PS 3.6
	@field UID_value        A C string containing the value of the UID
	@field UID_name         A C string containing the descritpion of the UID
	@field UID_type         The of the UID - one of the values in the DTkUIDType enumeration
	@field UID_standardPart A referencee to the part of the standard the UID is defined in
	@field UID_status       Indicates whether the UID is active or retired
*/
struct DTkUIDInfoRecElem
{
	char			* UID_value;
	char 			* UID_name;
	DTkUIDType		  UID_type;
	UInt32			  UID_standardPart;
	DTkElementStatus  UID_status;
};
typedef struct DTkUIDInfoRecElem DTkUIDInfoRecElem;
typedef DTkUIDInfoRecElem * DTkUIDInfoRecElemPtr;

#pragma mark struct DTkStandardSOPClassElem
/*!
    @typedef DTkStandardSOPClassElem
	@abstract Type definition of a structure containing information from the Standard SOP class table in section PS 3.4.B.5
	@field SOPClassUID     A C string containing the value UID of the SOP class
	@field SOPClassName    A C string containing the descritpion of the SOP class
	@field SOPClassCIODref A reference to the part of the standard the SOP class is defined in
	@field SOPClassStatus  Indicates whether the SOP class is active or retired
*/
struct DTkStandardSOPClassElem
{
	char			* SOPClassUID;
	char 			* SOPClassName;
	char 			* SOPClassCIODref;
	DTkElementStatus  SOPClassStatus;
};
typedef struct DTkStandardSOPClassElem DTkStandardSOPClassElem;
typedef DTkStandardSOPClassElem * DTkStandardSOPClassElemPtr;

#pragma mark -
#pragma mark enum DTkColorSpace
/*!
    @enum DTkColorSpace
	@abstract Enumeration color space types private to DICOM toolkit
	@constant kDTkColorSpaceMixed    Specifies a color space in which there are two
                                     distinct ranges of pixel values: a gray scale range that
                                     is subjected to the gray scale visualizaion pipe line, and an
                                     indexed color range where values are mapped by Palette Color LUTs.
                                     (section PS 3.3.C.8.13.3.1.2.1 of DICOM Standard)
	@constant kDTk32MixedPixelFormat A synonym of kDTkColorSpaceMixed that refers to a pixel format.
*/
typedef enum
{
	kDTkColorSpaceMixed = FOUR_CHAR_CODE('DMXD'),
	kDTk32MixedPixelFormat  = kDTkColorSpaceMixed
} DTkColorSpace;

#pragma mark struct DTkUInt8Vect_256
/*!
    @typedef DTkVOIScale_256
	@abstract Type definition of a structure containing a vector with 256 elements of type UInt8
	@field vectUInt8256  The vector
*/
struct DTkUInt8Vect_256
{
	UInt8 vectUInt8[256];
};
typedef struct DTkUInt8Vect_256 DTkUInt8Vect_256;
typedef DTkUInt8Vect_256 * DTkUInt8Vect_256Ptr;

#pragma mark -
#pragma mark  Support for DTkCodec[Set|Get]MaskParams 

#pragma mark struct DTkMaskDisplacementRec
/*!
    @typedef DTkMaskDisplacementRec
    @abstract Decleration of mask displacement record (in pixels).
    @field xDisp Horizontal displacement 
    @field yDisp Vertical displacement 
*/
typedef struct DTkMaskDisplacementRec
{
	Float32 xDisp;
	Float32 yDisp;
} DTkMaskDisplacementRec, *DTkMaskDisplacementRecPtr;


#pragma mark struct DTkMaskUserDefinedSrcDifRangeRec
/*!
    @typedef DTkMaskUserDefinedSrcDifRangeRec
    @abstract Decleration of user defined source and source-mask ranges (to be used for appropriately scaling the destination pixels)
    @discussion The values passed via this structure would probably be obtained by a preflight operation and should represent 
                the optimal compromise between individual image quality and frame-to-frame uniformity.
    @field usrSrcValueMin min source pixel value 
    @field usrSrcValueMax max source pixel value 
    @field usrDifValueMin min (source - mask) difference
    @field usrDifValueMax max (source - mask) difference
*/
typedef struct DTkMaskUserDefinedSrcDifRangeRec
{
	UInt16 usrSrcValueMin;
	UInt16 usrSrcValueMax;
	UInt16 usrDifValueMin;
	UInt16 usrDifValueMax;
} DTkMaskUserDefinedSrcDifRangeRec, *DTkMaskUserDefinedSrcDifRangeRecPtr;

#pragma mark enum DTkMaskFlags
/*!
	@enum DTkMaskFlags
	@abstract Enumeration of flags that influence te mask operation. Passed via the DTkCodecSetMaskParams with the 
	          kDTkMaskParamSelector_MaskFlags selector.
	@constant kDTkMaskFlag_ExpandDestToFullRange When this flag is set the mask operation expands the destination data 
	                                             to the full output range.
*/
typedef enum
{
	kDTkMaskFlag_ExpandDestToFullRange = 1 << 0
} DTkMaskFlags;

#pragma mark enum DTkMaskMethod
/*!
    @enum DTkMaskMethod
	@abstract Enumeration of supported mask methods
    @constant kDTkMaskMethod_UseExtremeRange The range of the intermediate difference pixel map is set to [sourceMax - maskMin, sourceMin - maskMax].
    @constant kDTkMaskMethod_UseExactRange The range of the intermediate difference pixel map is calculated on the fly from the actual source and mask pixel values.
    @constant kDTkMaskMethod_UseUserDefinedSrcRange The range of the intermediate difference pixel map is provided by 
                                                    the user by a call to DTkCodecSetMaskParams with the kDTkMaskParamSelector_UserDefinedSrcMskRange selector.
    @constant kDTkMaskMethod_UseAbsoluteDiff The destination pixel map is constructed using an absolute difference beteen source and mask pixel values.
    @constant kDTkMaskMethod_Unknown Specifies an unknown mask method (default).
*/
typedef enum
{
	kDTkMaskMethod_UseExtremeRange,
	kDTkMaskMethod_UseExactRange,	
	kDTkMaskMethod_UseUserDefinedSrcRange,
	kDTkMaskMethod_UseAbsoluteDiff,
	kDTkMaskMethod_Unknown = kDTkUnknown
} DTkMaskMethod;

#pragma mark enum DTkMaskParamSelector
/*!
    @enum DTkMaskParamSelector
	@abstract Enumeration of the allowed values for the selector parameter of the DICOM_CDSetMaskParams API function.
	@constant kDTkMaskParamSelector_MaskPixMap      A handle of a pixelmap to be used for the masking operation.
	@constant kDTkMaskParamSelector_DisplacementRec A record of type DTkMaskDisplacementRec. If not set the mask displacement defaults to (0,0).
	@constant kDTkMaskParamSelector_MaskMethod		Specifies the masking method to be used. 
													Use one of the values of the DTkMaskMethod enumeration.
													Defaults to kDTkMaskMethod_UseExactRange.
	@constant kDTkElemRecTypeSel_Unknown            Specifies an unknown record type.
*/
typedef enum 
{
	kDTkMaskParamSelector_MaskPixMap,
	kDTkMaskParamSelector_DisplacementRec,
	kDTkMaskParamSelector_MaskMethod,
	kDTkMaskParamSelector_MaskFlags,
	kDTkMaskParamSelector_UserDefinedSrcMskRange,	
	kDTkMaskParamSelector_Unknown = kDTkUnknown
} DTkMaskParamSelector;

#pragma mark -
#pragma mark  Support for Overlays 
#pragma mark enum DTkActivePixelMode
/*!
    @enum DTkActivePixelMode
    @abstract The constants defined in this enumeration are to be used with the toolkit's  
              DTkGraphicsImporter[S|G]etActivePixelMode functions and to specify the semantics
              of QuickTime's GraphicsImport[S|G]etImageIndex and GraphicsImportGetImageCount functions.
    @constant kDTkActivePixelModeImage When this constant is passed to DTkGraphicsImporterSetActivePixelMode QuickTime's
                                    image index calls refer to an DICOM file's image data.
    @constant kDTkActivePixelModeOverlay When this constant is passed to DTkGraphicsImporterSetActivePixelMode QuickTime's 
                                      image index calls refer to an DICOM file's overlay data.
	@discussion In addition to or instead of image data DICOM files may have overlay data. To specify which to use 
	            with the graphics importer's miscellaneous functions one may specify a context by using the
	            DTkGraphicsImporterSetActivePixelMode call, which uses the constants defined here.
*/
typedef enum
{
	kDTkActivePixelModeImage,
	kDTkActivePixelModeOverlay
} DTkActivePixelMode, *DTkActivePixelModePtr;



#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint reset
    #ifdef __DICOM_TOOLKIT_TYPES__RESTORE_TWOBYTEINTS
        #pragma fourbyteints off
    #endif
#elif PRAGMA_ENUM_OPTIONS
    #pragma option enum=reset
#elif defined(__DICOM_TOOLKIT_TYPES__RESTORE_PACKED_ENUMS)
    #pragma options(pack_enums)
#endif

#if PRAGMA_STRUCT_ALIGN
    #pragma options align=reset
#elif PRAGMA_STRUCT_PACKPUSH
    #pragma pack(pop)
#elif PRAGMA_STRUCT_PACK
    #pragma pack()
#endif

#ifdef PRAGMA_IMPORT_OFF
#pragma import off
#elif PRAGMA_IMPORT
#pragma import reset
#endif

#ifdef __cplusplus
}
#endif


#endif // __DICOM_TOOLKIT_TYPES__

